class SBFactoryMonsterCounter extends Mutator config(SBFactoryMonsterCounter);


enum ELogType {
	LOG_None,
	LOG_Error,
	LOG_Warning,
	LOG_Info,
	LOG_Verbose,
	LOG_Debug,
	LOG_All,
};


struct DontTreatThisFactory {
	var string MapName, FactoryName;
};

var config DontTreatThisFactory DontTreatTheseFactories[32];
var config bool bActive;
var config bool bUseLogTimeStamp; // Add a time stamp in front of every log message?
var config string CountMessage;
var config string CompleteMessage;
var config byte MinCapacity; // factories with less capacity are ignored
var config ELogType LogLevel;



/************************** internal variables **************************/
const CMutatorVersion = 1;


static function bool CharIsWhite(int charcode) {
/******************************************************************************
Returns TRUE if the char with *charcode* is a white char (= no ink is used if
printed).
******************************************************************************/

	switch (charcode) {
		case 32: // Space
			break;
		case 13: // CR
			break;
		case 10: // LF
			break;
		case 9: // tab
			break;
		case 9: // tab
			break;
		case 12: // FF
			break;
		case 11: // vTab
			break;
		default:
			return false;
	}
	return true;
}



function bool FindCounterByTag(name CounterTag, out Counter FoundCounter, out int CounterCount) {
local Counter FirstFound;

	CounterCount = 0;
	foreach AllActors(class'Counter', FoundCounter, CounterTag)
	{
		if (FirstFound == None)
			FirstFound = FoundCounter;
		CounterCount++;
		logger(LOG_Debug, "FindCounterByTag", FoundCounter @ "found by Tag" @ string(CounterTag));
	}
	FoundCounter = FirstFound;
	if (CounterCount > 0)
	{
		return true;
	}
	else
	{
		logger(LOG_Debug, "FindCounterByTag", "no counter found by Tag" @ string(CounterTag));
		return false;
	}
}



function bool FindSBCounterByTag(name CounterTag) {
local SBCounter SBCntr;

	foreach AllActors(class'SBCounter', SBCntr, CounterTag)
		if (SBCntr != None)
			return true;
	return false;
}



function string GetCompleteMessage(string OldCompleteMessage, ThingFactory TF) {
	Return ReplaceVar(OldCompleteMessage, "%p", GetPawnName(class <Pawn>(TF.Prototype)));
}



function string GetCountMessage(string OldCountMessage, ThingFactory TF) {
	Return ReplaceVar(OldCountMessage, "%p", GetPawnName(class <Pawn>(TF.Prototype)));
}



function string GetPawnName(class<Pawn> P) {
/******************************************************************************
Returns Pawn.MenuName. If that is empty, the class name is returned.
******************************************************************************/

	if (P.Default.MenuName == "")
		return string(P.Default.Class.Name);
	else
		return P.Default.MenuName;
}



function string GetTimeStampStr() {
/******************************************************************************
Returns a formatted time string YYYY-MM-DD hh:mm:ss
******************************************************************************/
	return Level.Year $ "-" $ TwoDigitString(Level.Month) $ "-"
		$ TwoDigitString(Level.Day) $ " " $ TwoDigitString(Level.Hour) $ ":"
		$ TwoDigitString(Level.Minute) $ ":" $ TwoDigitString(Level.Second);
}



function bool Init() {
local string MutatorInfoStr;
local byte i;

	MutatorInfoStr = "Mutator" @ self.class.name @ "Version" @ CMutatorVersion @ "with LogLevel" @ GetEnum(Enum'ELogType', LogLevel);

	if (LogLevel == LOG_All)
	{
		Logger(LOG_All, "Init", "--- INI settings start ---");
		for (i = 0; i < ArrayCount(DontTreatTheseFactories); i++)
		{
			Logger(LOG_All, "Init", "DontTreatTheseFactories[" $ i $ "].MapName=\"" $ DontTreatTheseFactories[i].MapName $ "\", DontTreatTheseFactories[" $ i $ "].FactoryName=\"" $ DontTreatTheseFactories[i].FactoryName $ "\"");
		}
		Logger(LOG_All, "Init", "bActive=" $ bActive);
		Logger(LOG_All, "Init", "CountMessage='" $ CountMessage $ "'");
		Logger(LOG_All, "Init", "CompleteMessage='" $ CompleteMessage $ "'");
		Logger(LOG_All, "Init", "MinCapacity=" $ MinCapacity);
		Logger(LOG_All, "Init", "--- INI settings end ---");
	}


	if(bActive)
	{
		Logger(LOG_Info, "Init", MutatorInfoStr @ "loaded");
		return true;
	}
	else
	{
		Logger(LOG_Info, "Init", MutatorInfoStr @ "disabled via INI settings");
		return false;
	}
}



function Logger(ELogType NeededLogLevel, string FunctionName, string msg) {
	if (LogLevel < NeededLogLevel)
		return;
	LoggerDirect(NeededLogLevel, FunctionName, msg);
}



function LoggerDirect(ELogType NeededLogLevel, string FunctionName, string msg) {
// issues the message without respect to *NeededLogLevel*
local string TimeStampStr;

	if (bUseLogTimeStamp)
		TimeStampStr = GetTimeStampStr() $ " ";
	Log(TimeStampStr $ self $ "." $ FunctionName @ GetEnum(Enum'ELogType', NeededLogLevel) $ ":" @ msg);
}



function int PocessFactories(out int CreatedCounterNum, out int ModifiedCounterNum) {
/*******************************************************************************
Returns the number of spawned counters.
*******************************************************************************/
local ThingFactory TF;
local Counter Cntr;
local int CounterCount;
local SBCounter NewCounter;
local bool bModified;
local byte DontTreatTheseFactoriesIndex;

	foreach AllActors(class'ThingFactory', TF)
	{
		Logger(LOG_All, "PocessFactories", "processing" @ TF $ ", Prototype=" $ TF.ProtoType $ ", Tag='" $ TF.Tag $ "'");
		if (TreatThisFactory(Level.Outer.Name, TF, DontTreatTheseFactoriesIndex))
		{
			if (TF.Capacity >= MinCapacity)
			{
				if (ClassIsChildOf(TF.ProtoType, class'Pawn'))
				{
					if (FindSBCounterByTag(TF.Tag))
					{
						Logger(LOG_Debug, "PocessFactories", "SBCounter with Tag='" $ TF.Tag $ "' already exists, skipping");
					}
					else
					{
						if (FindCounterByTag(TF.Tag, Cntr, CounterCount))
						{
							logger(LOG_Debug, "PocessFactories", Cntr @ "found related to" @ TF);
							bModified = false;
							if (Trim(Cntr.CountMessage) ~= "" || trim(Cntr.CountMessage) ~= Trim(Cntr.Default.CountMessage))
							{
								Cntr.CountMessage = GetCountMessage(CountMessage, TF);
								bModified = true;
								logger(LOG_Debug, "PocessFactories", "empty/default message of" @ Cntr @ "set to '" $ Cntr.CountMessage $ "'");
							}
							if ( ! Cntr.bShowMessage)
							{
								Cntr.bShowMessage = true;
								bModified = true;
								logger(LOG_Debug, "PocessFactories", Cntr $ ".bShowMessage changed to TRUE");
							}
							if (bModified)
								ModifiedCounterNum++;
						}
						else
						{
							NewCounter = Spawn(class'SBCounter', ,TF.Tag);
							if (NewCounter != None)
							{
								NewCounter.Prototype = TF.Prototype;
								NewCounter.NumToCount = TF.Capacity;
								NewCounter.bShowMessage = true;
								NewCounter.CountMessage = GetCountMessage(CountMessage, TF);
								NewCounter.CompleteMessage = GetCompleteMessage(CompleteMessage, TF);
								if (LogLevel >= LOG_Debug)
									logger(LOG_Debug, "PocessFactories", NewCounter @ "created with NumToCount=" $ NewCounter.NumToCount $ ", CountMessage=\"" $ NewCounter.CountMessage $ "\", CompleteMessage=\"" $ NewCounter.CompleteMessage $ "\"");
								CreatedCounterNum++;
							}
							else
								logger(LOG_Warning, "PocessFactories", "Could not spawn a counter related to" @ TF);
						}
					}
				}
				else
					Logger(LOG_Debug, "PocessFactories", "Factorie's Prototype is not a child of Class'Pawn', skipping");
			}
			else
				logger(LOG_Debug, "PocessFactories", "factory" @ TF @ "ignored because its capacity of" @ TF.Capacity @ "is lower than MinCapacity=" $ MinCapacity);
		}
		else
			logger(LOG_Verbose, "PocessFactories", "factory" @ TF @ "ignored because it is in list *DontTreatTheseFactories* at line" @ DontTreatTheseFactoriesIndex);
	}
	return CounterCount;
}



event PostBeginPlay() {
local int CreatedCounterNum, ModifiedCounterNum;
local byte MapIgnoreFoundIndex;

	Super.PostBeginPlay();
	if (Init())
	{
		if ( ! TreatThisMap(Level.Outer.Name, MapIgnoreFoundIndex))
		{
			Logger(LOG_Verbose, "PostBeginPlay", "map" @ string(Level.Outer.Name) @ "found in list 'DontTreatTheseFactories' at line" @ MapIgnoreFoundIndex $ " with empty 'FactoryName' => doing nothing");
			return;
		}
		PocessFactories(CreatedCounterNum, ModifiedCounterNum);
		Logger(LOG_Info, "PostBeginPlay", CreatedCounterNum @ "counters created," @ ModifiedCounterNum @ "existing counters modified");
	}
}



function function string ReplaceVar(string s, string OutString, string InString) {
local int i;

LoopAgain:
	i = InStr(s, OutString);
	if (i >= 0)
	{
		s = Left(s, i) $ InString $ Mid(s, i + Len(OutString));
		goto LoopAgain;
	}
	Return s;
}



function bool TreatThisFactory(coerce string MapFileName, ThingFactory TF, out byte FoundIndex) {

	Logger(LOG_All, "TreatThisFactory", TF @ "is checked now if it should be treated for map" @ MapFileName);
	for (FoundIndex = 0; FoundIndex < ArrayCount(DontTreatTheseFactories); FoundIndex++)
	{
		if (DontTreatTheseFactories[FoundIndex].MapName == "" && string(TF.Class.Name) ~= DontTreatTheseFactories[FoundIndex].FactoryName)
		{
			Logger(LOG_Debug, "TreatThisFactory", TF @ "is not treated because its class name matches entry #" $ FoundIndex @ "in INI file");
			return false;
		}
		if (MapFileName ~= DontTreatTheseFactories[FoundIndex].MapName && DontTreatTheseFactories[FoundIndex].FactoryName ~= string(TF.Name))
			return false;
	}
	return true;
}



function bool TreatThisMap(coerce string MapFileName, optional out byte FoundIndex) {
	for (FoundIndex = 0; FoundIndex < ArrayCount(DontTreatTheseFactories); FoundIndex++)
		if (MapFileName ~= DontTreatTheseFactories[FoundIndex].MapName && DontTreatTheseFactories[FoundIndex].FactoryName == "")
			return false;
	return true;
}



static function string Trim(string s) {
/******************************************************************************
Returns both sides timmed string *s*
******************************************************************************/
	Return TrimL(TrimR(s));
}



static function string TrimL(string s) {
/******************************************************************************
Returns left side timmed string *s*
******************************************************************************/
local int i;

	while (i < len(s) && CharIsWhite(Asc(Mid(s, i, 1))))
		i++;
	return Right(s, Len(s) - i);
}



static function string TrimR(string s) {
/******************************************************************************
Returns right side timmed string *s*
******************************************************************************/
local int i;

	i = Len(s);
	while (i > 0 && CharIsWhite(Asc(Mid(s, i - 1, 1))))
		i--;
	return Left(s, i);
}



static function string TwoDigitString(int Num) {
	if (Num >= 10)
		return string(Num);
	else
		return "0" $ string(Num);
}



defaultproperties {
	bActive=true
	bUseLogTimeStamp=true
	CountMessage="Only %i more %p to go..."
	CompleteMessage="%ps are history now!"
	LogLevel=LOG_Info
	MinCapacity=2
}
